 /* *******************************************************************************
 // @file    startup_MDR1206.S
 // @author  Milandr Application Team
 // @version V1.1.0
 // @date    04/09/2024
 // @brief   Core device startup file for MDR1206.
 // *******************************************************************************
 // <br><br>
 //
 // THE PRESENT FIRMWARE IS FOR GUIDANCE ONLY. IT AIMS AT PROVIDING CUSTOMERS
 // WITH CODING INFORMATION REGARDING MILANDR'S PRODUCTS IN ORDER TO FACILITATE
 // THE USE AND SAVE TIME. MILANDR SHALL NOT BE HELD LIABLE FOR ANY
 // DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING
 // FROM THE CONTENT OF SUCH FIRMWARE AND/OR A USE MADE BY CUSTOMERS OF THE
 // CODING INFORMATION CONTAINED HEREIN IN THEIR PRODUCTS.
 //
 // <h2><center>&copy; COPYRIGHT 2024 Milandr</center></h2>
 // ********************************************************************************/

#include "system_MDR32VF0xI_config.h"

#if defined ( __GNUC__ ) // GCC compiler
#if defined ( USE_MDR1206FI ) || defined ( USE_MDR1206AFI ) || defined ( USE_MDR1206 ) // MCU definition

#if !defined(CSR_MTVT)
#define CSR_MTVT       0x307
#endif
#if !defined(CSR_MCLICBASE)
#define CSR_MCLICBASE  0x308
#endif

#if __riscv_xlen == 64
    #define LREG           ld
    #define SREG           sd
    #define REGBYTES       8
    #define LOG2_REGBYTES  3
    #define XWORD          .dword
#else
    #define LREG           lw
    #define SREG           sw
    #define REGBYTES       4
    #define LOG2_REGBYTES  2
    #define XWORD          .word
#endif

// Bootloader revision ----------------------
#define BOOT_REV_MAJOR  (1)
#define BOOT_REV_MINOR  (1)
#define BOOT_REV_PATCH  (0)

#define BOOT_REV_MAJOR_pos  (16)
#define BOOT_REV_MINOR_pos  (8)
#define BOOT_REV_PATCH_pos  (0)
// ------------------------------------------

// ------------------------------------------
.section ".text.init"
.globl _start
_start:
    j _start_real

    .align (2) // Boot revision by semver standard . Please, keep origin this at 2-nd word.
    .word (BOOT_REV_MAJOR << BOOT_REV_MAJOR_pos) | (BOOT_REV_MINOR << BOOT_REV_MINOR_pos) | (BOOT_REV_PATCH << BOOT_REV_PATCH_pos)

_start_real:
// Boot statrt. Setup CLIC Trap.
    la t0, Trap_IRQHandler_real
    csrs mtvec, t0

// At first time config PD11 as output with hi-level.
// Enable clock for RSTCLK, PORTD and FLASH.
    // MDR_RST_CLK->PER2_CLOCK = (0x00000010 | RST_CLK_PER2_CLOCK_PCLK_EN_BKP | RST_CLK_PER2_CLOCK_PCLK_EN_PORTD);
    lui a5,0x40001
    add a5,a5,16 // 40001010
    lui a4,0x50020
    sw a5,28(a4)

// Config PD11 as digital out whith hi-level.
    lui a5,0x500f0
    bset a4,zero,0x16
    // MDR_PORTD->PWR   = 1<<11;
    sw a4,24(a5)
    // MDR_PORTD->SETTX = 1<<11;
    bset a4,zero,0xb
    sw a4,32(a5)
    // MDR_PORTD->ANALOG= 1<<11;
    sw a4,12(a5)
    // MDR_PORTD->OE    = 1<<11;
    sw a4,4(a5)

// Setup core clock from HSI.
    // MDR_RST_CLK->CPU_CLOCK=0;
    lui s1,0x50020
    sw zero,12(s1) // 5002000c

//  Zeroing registers
    li x1, 0
//    li x2, 0 (sp)init in code below
    li x3, 0
    li x4, 0
//    li x5, 0 (t0)init in code above
    li x6, 0
    li x7, 0
    li x8, 0
//    li x9, 0 (s1)init in code above
    li x10,0
    li x11,0
    li x12,0
    li x13,0
//    li x14,0 (a4)init in code above
//    li x15,0 (a5)init in code above
    li x16,0
    li x17,0
    li x18,0
    li x19,0
    li x20,0
    li x21,0
    li x22,0
    li x23,0
    li x24,0
    li x25,0
    li x26,0
    li x27,0
    li x28,0
    li x29,0
    li x30,0
    li x31,0

    /*-------------
    //  Clean RAM
    //-------------*/
/*
    la sp, __data_start
    la a1, __stack_top
    bgeu a1, a2, clean_ram_loop_end
clean_ram_loop:
    sw x0,0(sp)
    sw x0,4(sp)
    sw x0,8(sp)
    sw x0,12(sp)
    sw x0,16(sp)
    sw x0,20(sp)
    sw x0,24(sp)
    sw x0,28(sp)
    addi sp, sp, 4*8
    bltu sp, a1, clean_ram_loop
clean_ram_loop_end:
*/

    /*-------------
    //  Copy data to RAM
    //-------------*/
/*
    la a0, __data_load_start
    la a1, __data_start
    la a2, __data_end
    beq  a0, a1, copy_data_loop_end           // do i need to copy?
    bgeu a1, a2, copy_data_loop_end
copy_data_loop:
    lw t0, (a0)
    sw t0, (a1)
    addi a0, a0, 4
    addi a1, a1, 4
    bltu a1, a2, copy_data_loop
copy_data_loop_end:
*/
    /* Set stack pointer to the end of available memory*/
    la sp, __stack_top

    /*-----------
    // Call main
    //-----------*/
    call main
    /* get main return code*/
    mv t0, a0
    j .
// ------------------------------------------

// ------------------------------------------
// Must be aligned by 64-byte boundary (2^6).
.align 6
Trap_IRQHandler_real:
    // Setup BKP clock.
    // MDR_RST_CLK->PER2_CLOCK |= (RST_CLK_PER2_CLOCK_PCLK_EN_BKP);
    lui a4,0x50020
    lw a5,28(a4)
    bset a5,a5,0xc
    sw a5,28(a4)

    // Increment fault counter.
    // MDR_BKP->WPR = 0x8555AAA1;
    lui a5,0x8555b
    add a5,a5,-1375 // 8555aaa1
    lui a4,0x50060
    sw a5,0(a4)

    // MDR_BKP->MEM[0]++;
    lui a3,0x50061
    lw a5,0(a3)
    add a5,a5,1
    sw a5,0(a3)

    // MDR_BKP->WPR = 0;
    sw zero,0(a4) // 50060000

    la a4, _start_real
    csrw mepc, a4

    mret      	// Another way when exception is occurred - return to adress from MEPC.

    j .				// Emergency deadlock. In normal this code must be unreachable.
// ------------------------------------------

#endif // USE_MDR1206
#endif // __GNUC__
